/***************************************************************************
                              systraymanager.cpp
                             --------------------
    begin                : 04.03.2003
    copyright            : (C) 2003 by Tamas Bara
    email                : tbara@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "systray4jd.h"
#include "systraymanager.h"
#include "st4jevent.h"
#include "menubase.h"
#include "systraymenu.h"
#include "submenu.h"
#include "transceiver.h"

extern const char* penguin_xpm[];

QPixmap* SysTrayManager::s_pPenguin = 0;

SysTrayManager::SysTrayManager( Transceiver* pTransceiver )
{
    m_pTransceiver = pTransceiver;

    if( !s_pPenguin ) s_pPenguin = new QPixmap( penguin_xpm );

    m_menus.setAutoDelete( true );
    m_icons.setAutoDelete( true );
}

SysTrayManager::~SysTrayManager()
{
}

void SysTrayManager::customEvent( QCustomEvent* pCustomEvent )
{
    if( pCustomEvent->type() == SysTray4JavaEvent::SYSTRAY4J_EVENT )
    {
        SysTray4JavaEvent* pEvent = ( SysTray4JavaEvent* ) pCustomEvent;
        MenuBase* pMenu = 0;
        QPixmap* pPixmap = 0;
        int menuId = -1;

        if( pEvent->getToDo() == Transceiver::ADD_MAINMENU )
        {
            pPixmap = getIcon( pEvent->getIconFileName() );

            pMenu = new SysTrayMenu(
                pEvent->getMenuId(), m_pTransceiver, this, *pPixmap, pEvent->getToolTip() );

            m_menus.append( pMenu );
        }
        else if( pEvent->getToDo() == Transceiver::ADD_SUBMENU )
        {
            pMenu = new SubMenu( pEvent->getMenuId(), m_pTransceiver, this );

            m_menus.append( pMenu );
        }
        else if( pEvent->getToDo() == Transceiver::SET_TOOLTIP )
        {
            pMenu = findMenu( pEvent->getMenuId() );
            pMenu->setToolTip( pEvent->getToolTip() );
        }
        else if( pEvent->getToDo() == Transceiver::SET_ICON )
        {
            pMenu = findMenu( pEvent->getMenuId() );
            pPixmap = getIcon( pEvent->getIconFileName() );
            pMenu->setIcon( *pPixmap );
        }
        else if( pEvent->getToDo() == Transceiver::SHOW_ICON )
        {
            pMenu = findMenu( pEvent->getMenuId() );
            pMenu->showIcon( pEvent->getShowIcon() );
        }
        else if( pEvent->getToDo() == Transceiver::ADD_ITEM )
        {
            menuId = pEvent->getMenuId();
            pMenu = findMenu( menuId );
            pMenu->addItem( pEvent->getItemIndex(),
                            pEvent->getItemLabel(),
                            pEvent->getItemCheckable(),
                            pEvent->getItemCheck(),
                            pEvent->getItemEnable() );
            
        }
        else if( pEvent->getToDo() == Transceiver::ENABLE_ITEM )
        {
            menuId = pEvent->getMenuId();
            pMenu = findMenu( menuId );
            pMenu->enableItem( pEvent->getItemIndex(), pEvent->getItemEnable() );
        }
        else if( pEvent->getToDo() == Transceiver::CHECK_ITEM )
        {
            menuId = pEvent->getMenuId();
            pMenu = findMenu( menuId );
            pMenu->checkItem( pEvent->getItemIndex(), pEvent->getItemCheck() );
        }
        else if( pEvent->getToDo() == Transceiver::SET_ITEMLABEL )
        {
            menuId = pEvent->getMenuId();
            pMenu = findMenu( menuId );
            pMenu->setItemLabel( pEvent->getItemIndex(), pEvent->getItemLabel() );
            
        }
        else if( pEvent->getToDo() == Transceiver::REMOVE_ITEM )
        {
            menuId = pEvent->getMenuId();
            pMenu = findMenu( menuId );
            pMenu->removeItem( pEvent->getItemIndex() );
            
        }
        else if( pEvent->getToDo() == Transceiver::REMOVE_ALL )
        {
            menuId = pEvent->getMenuId();
            pMenu = findMenu( menuId );
            pMenu->removeAll();
        }
    }
}

QPixmap* SysTrayManager::getIcon( QString& iconFileName )
{
    QPixmap* pPixmap = m_icons.find( iconFileName );
    if( !pPixmap )
    {
        pPixmap = new QPixmap( iconFileName );
        if( !pPixmap->isNull() )
        {
            m_icons.insert( iconFileName, pPixmap );
        }
        else pPixmap = s_pPenguin;
    }

    return pPixmap;
}

MenuBase* SysTrayManager::findMenu( int menuId )
{
    MenuBase* pMenu = 0;
    for( pMenu = m_menus.first(); pMenu; pMenu = m_menus.next() )
    {
        if( pMenu->getId() == menuId ) break;
    }

    return pMenu;
}
