/***************************************************************************
                               systraymenu.cpp
                             -------------------
    begin                : 04.03.2003
    copyright            : (C) 2003 by Tamas Bara
    email                : tbara@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qtooltip.h>

#include "systraymenu.h"
#include "submenu.h"
#include "transceiver.h"
#include "systraymanager.h"

// represents a main menu

SysTrayMenu::SysTrayMenu( int id,                       // id of this menu
                          Transceiver* pTransceiver,    // transceiver reference
                          SysTrayManager* pManager,     // manager reference
                          QPixmap& pixmap,              // icon
                          QString& toolTip )            // tooltip
                          
    : MenuBase( id, pTransceiver, pManager )
{
    QToolTip::add( this, toolTip );

    setPixmap( pixmap );
    
    show();
}

void SysTrayMenu::mousePressEvent( QMouseEvent* e )
{
    if( e->button() == RightButton )
    {
        QPoint p;
        p.setX( e->globalX() - m_pMenu->sizeHint().width() );
        p.setY( e->globalY() - m_pMenu->sizeHint().height() );
        m_pMenu->exec( p );
    }
    else if( e->button() == LeftButton )
    {
        m_pTransceiver->sendIconClicked( m_id );
    }
}

void SysTrayMenu::mouseDoubleClickEvent( QMouseEvent* e )
{
    if( e->button() == LeftButton )
    {
        m_pTransceiver->sendIconDblClicked( m_id );
    }
}

void SysTrayMenu::setToolTip( QString& toolTip )
{
    QToolTip::add( this, toolTip );
}

void SysTrayMenu::setIcon( QPixmap& pixmap )
{
    setPixmap( pixmap );
}

void SysTrayMenu::showIcon( bool show )
{
    if( show ) this->show();
    else hide();
}

void SysTrayMenu::addItem( int id, QString& label, bool checkable, bool checked, bool enabled )
{
    SubMenu* psMenu = 0;

    // check whether the label refers to a submenu
    if( label.startsWith( "#SUB<" ) && label.contains( "><" ) && label.endsWith( ">" ) )
    {
        // submenu reference string format:
        //   #SUB<id><label>

        // extract the id of the submenu
        int begin = label.find( "<" ) + 1;
        QString strId = label.mid( begin, label.find( ">" ) - begin );

        // extract the label of the submenu
        begin = label.find( "><" ) + 2;
        label = label.mid( begin, label.length() - begin - 1 );

        int menuId = strId.toInt();
        psMenu = ( SubMenu* ) m_pManager->findMenu( menuId );
    }

    int pos = m_pMenu->count() - id;
    if( psMenu ) m_pMenu->insertItem( label, psMenu->getPopupMenu(), id, pos );
    else if( label != "#SEP" ) m_pMenu->insertItem( label, id, pos );
    else m_pMenu->insertSeparator( pos );

    int newId = id + 1;
    for( int i = pos - 1; i > -1; i-- ) m_pMenu->setId( i, newId++ );

    if( checkable )
    {
        m_checkableItems.append( id );
        if( checked ) checkItem( id, checked );
    }
    
    if( !enabled ) m_pMenu->setItemEnabled( id, enabled );
}
