/***************************************************************************
                               transceiver.cpp
                             -------------------
    begin                : 04.03.2003
    copyright            : (C) 2003 by Tamas Bara
    email                : tbara@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qstring.h>

#include "transceiver.h"
#include "st4jevent.h"
#include "systraymanager.h"

#ifdef ST4JDEBUG_MODE
#include <iostream>
using namespace std;
#endif

char** Transceiver::MESSAGES = 0;

Transceiver::Transceiver( KSocket* pSocket )
{
    if( !MESSAGES ) init();

    m_pSocket = pSocket;

    m_pFile = new QFile();
    m_pFile->open( IO_ReadWrite, pSocket->socket() );

    m_stream.setDevice( m_pFile );

    m_pManager = new SysTrayManager( this );

    start();
}

Transceiver::~Transceiver()
{
    delete m_pManager;
    delete m_pSocket;
    delete m_pFile;
}

void Transceiver::init()
{
    MESSAGES = new char*[ NUM_MESSAGES ];
    MESSAGES[ ICON_CLICKED ]       = strdup( "ICON CLICKED" );
    MESSAGES[ ICON_DBLCLICKED ]    = strdup( "ICON DBLCLICKED" );
    MESSAGES[ ITEM_SELECTED ]      = strdup( "ITEM SELECTED" );
    MESSAGES[ ADD_MAINMENU ]       = strdup( "ADD MAINMENU" );
    MESSAGES[ ADD_SUBMENU ]        = strdup( "ADD SUBMENU" );
    MESSAGES[ SET_ICON ]           = strdup( "SET ICON" );
    MESSAGES[ SHOW_ICON ]          = strdup( "SHOW ICON" );
    MESSAGES[ SET_TOOLTIP ]        = strdup( "SET TOOLTIP" );
    MESSAGES[ ADD_ITEM ]           = strdup( "ADD ITEM" );
    MESSAGES[ ENABLE_ITEM ]        = strdup( "ENABLE ITEM" );
    MESSAGES[ CHECK_ITEM ]         = strdup( "CHECK ITEM" );
    MESSAGES[ SET_ITEMLABEL ]      = strdup( "SET ITEMLABEL" );
    MESSAGES[ REMOVE_ITEM ]        = strdup( "REMOVE ITEM" );
    MESSAGES[ REMOVE_ALL ]         = strdup( "REMOVE ALL" );
}

void Transceiver::run()
{
  #ifdef ST4JDEBUG_MODE
    cout << "[" << this << "] **** transmission initiated" << endl;
  #endif

    QString request;
    QString line;
    request = m_stream.readLine();
    while( request )
    {
      #ifdef ST4JDEBUG_MODE
        cout << "[" << this << "] received: " << request << endl;
      #endif
      
        if( request == MESSAGES[ ADD_MAINMENU ] )       addMainMenu();
        else if( request == MESSAGES[ ADD_SUBMENU ] )   addSubMenu();
        else if( request == MESSAGES[ SET_TOOLTIP ] )   setTooltip();
        else if( request == MESSAGES[ SET_ICON ] )      setIcon();
        else if( request == MESSAGES[ SHOW_ICON ] )     showIcon();
        else if( request == MESSAGES[ ADD_ITEM ] )      addItem();
        else if( request == MESSAGES[ ENABLE_ITEM ] )   enableItem();
        else if( request == MESSAGES[ CHECK_ITEM ] )    checkItem();
        else if( request == MESSAGES[ SET_ITEMLABEL ] ) setItemLabel();
        else if( request == MESSAGES[ REMOVE_ITEM ] )   removeItem();
        else if( request == MESSAGES[ REMOVE_ALL ] )    removeAll();
        else
        {
          #ifdef ST4JDEBUG_MODE
            cout << "[" << this << "] **** fatal error: unknown message" << endl;
          #endif
            
            break;
        }

        request = m_stream.readLine();
    }

    delete this;

    #ifdef ST4JDEBUG_MODE
      cout << "[" << this << "] **** transmission finished" << endl;
    #endif  
}

void Transceiver::sendIconClicked( int menuId )
{
    m_stream << MESSAGES[ ICON_CLICKED ] << endl;
    m_stream << menuId << endl;
}

void Transceiver::sendIconDblClicked( int menuId )
{
    m_stream << MESSAGES[ ICON_DBLCLICKED ] << endl;
    m_stream << menuId << endl;
}

void Transceiver::sendItemSelected( int menuId, int index )
{
    m_stream << MESSAGES[ ITEM_SELECTED ] << endl;
    m_stream << menuId << endl;
    m_stream << index << endl;
}

void Transceiver::addMainMenu()
{
    QString line = m_stream.readLine();
    int menuId = line.toInt();

    QString iconFileName = m_stream.readLine();    
    QString toolTip = m_stream.readLine();

    SysTray4JavaEvent* pEvent =
        new SysTray4JavaEvent( ADD_MAINMENU, menuId, iconFileName, toolTip );

    postEvent( m_pManager, pEvent );
}

void Transceiver::addSubMenu()
{
    QString line = m_stream.readLine();
    int menuId = line.toInt();

    SysTray4JavaEvent* pEvent = new SysTray4JavaEvent( ADD_SUBMENU, menuId );

    postEvent( m_pManager, pEvent );
}

void Transceiver::setTooltip()
{
    QString line = m_stream.readLine();
    int menuId = line.toInt();

    QString toolTip = m_stream.readLine();
    
    SysTray4JavaEvent* pEvent = new SysTray4JavaEvent( SET_TOOLTIP, menuId, toolTip );

    postEvent( m_pManager, pEvent );
}

void Transceiver::setIcon()
{
    QString line = m_stream.readLine();
    int menuId = line.toInt();

    QString iconFileName = m_stream.readLine();

    SysTray4JavaEvent* pEvent = new SysTray4JavaEvent( SET_ICON, menuId, iconFileName );

    postEvent( m_pManager, pEvent );
}

void Transceiver::showIcon()
{
    QString line = m_stream.readLine();
    int menuId = line.toInt();

    line = m_stream.readLine();
    bool showIcon = line == "true";

    SysTray4JavaEvent* pEvent = new SysTray4JavaEvent( SHOW_ICON, menuId, showIcon );

    postEvent( m_pManager, pEvent );
}

void Transceiver::addItem()
{
    QString line = m_stream.readLine();
    int menuId = line.toInt();
    
    line = m_stream.readLine();
    int itemIndex = line.toInt();

    QString itemLabel = m_stream.readLine();

    line = m_stream.readLine();
    bool checkable = line == "true";

    line = m_stream.readLine();
    bool checked = line == "true";
    
    line = m_stream.readLine();
    bool enabled = line == "true";
    
    SysTray4JavaEvent* pEvent = new SysTray4JavaEvent( ADD_ITEM, menuId, itemIndex, itemLabel,
        checkable, checked, enabled );

    postEvent( m_pManager, pEvent );
}

void Transceiver::enableItem()
{
    QString line = m_stream.readLine();
    int menuId = line.toInt();
    
    line = m_stream.readLine();
    int itemIndex = line.toInt();
    
    line = m_stream.readLine();
    bool enable = line == "true";
    
    SysTray4JavaEvent* pEvent =
        new SysTray4JavaEvent( ENABLE_ITEM, menuId, itemIndex, enable );

    postEvent( m_pManager, pEvent );
}

void Transceiver::checkItem()
{
    QString line = m_stream.readLine();
    int menuId = line.toInt();

    line = m_stream.readLine();
    int itemIndex = line.toInt();

    line = m_stream.readLine();
    bool check = line == "true";

    SysTray4JavaEvent* pEvent =
        new SysTray4JavaEvent( CHECK_ITEM, menuId, itemIndex, check );

    postEvent( m_pManager, pEvent );
}

void Transceiver::setItemLabel()
{
    QString line = m_stream.readLine();
    int menuId = line.toInt();

    line = m_stream.readLine();
    int itemIndex = line.toInt();

    QString itemLabel = m_stream.readLine();

    SysTray4JavaEvent* pEvent =
        new SysTray4JavaEvent( SET_ITEMLABEL, menuId, itemIndex, itemLabel );

    postEvent( m_pManager, pEvent );
}

void Transceiver::removeItem()
{
    QString line = m_stream.readLine();
    int menuId = line.toInt();

    line = m_stream.readLine();
    int itemIndex = line.toInt();

    SysTray4JavaEvent* pEvent = new SysTray4JavaEvent( REMOVE_ITEM, menuId, itemIndex );

    postEvent( m_pManager, pEvent );
}

void Transceiver::removeAll()
{
    QString line = m_stream.readLine();
    int menuId = line.toInt();

    SysTray4JavaEvent* pEvent = new SysTray4JavaEvent( REMOVE_ALL, menuId );

    postEvent( m_pManager, pEvent );
}
